/*
K-LIFE game of life 5x8 LED ARRAY with PIC16F84A

Copyright (C) 2011 Krasimir Stefanov <lokiisyourmaster@gmail.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  ,         hex-
picprog --burn --input k-life.hex --pic /dev/ttyS0
,    : http://www.instructables.com/id/JDM2-based-PIC-Programmer/
*/

#include <htc.h>
#include "delay.c"

__CONFIG(0x3FF1);

unsigned int frame_length = 1000;
unsigned char chr[5];
unsigned char pattern = 0;
unsigned int generations = 0;

//function prototypes
void display_frame(void);
unsigned char gval(signed char x, signed char y);
void step(void);
void randomize_field(void);
void clear_field(void);
unsigned int my_rand(void);

void main(void){

	
	TRISA = 0x00;
	TRISB = 0x00;
	PORTA = 0b11111;
	PORTB = 0b00000000;

	clear_field();
	randomize_field();
	while (1){
		display_frame();
		step();
		generations++;
		if (generations>30) {
			generations=0;
			clear_field();
			randomize_field();
		}
	}
}

void step(void){
	unsigned char x,y,neighboors,total_live_cells,tmp;
	unsigned char _chr[5];
	
	total_live_cells = 0;
	for (x=0;x<5;x++){
		for (y=0;y<7;y++){	
			total_live_cells += (chr[x] >> y) & 0b00000001;
		}
	}
	
	if (total_live_cells<1) {
		clear_field();
		randomize_field();
	}

	for (tmp=0;tmp<5;tmp++){
		_chr[tmp] = chr[tmp];
	}
	
	for (x=0;x<5;x++){
		for (y=0;y<7;y++){
			neighboors = gval(x-1,y-1)+gval(x,y-1)+gval(x+1,y-1)+gval(x-1,y)+gval(x+1,y)+gval(x-1,y+1)+gval(x,y+1)+gval(x+1,y+1);
			if (neighboors<2){
				//   ,  -  2    
				_chr[x] &= ~(0b00000001 << y);
			} else if(neighboors>3){
				//       3    
				_chr[x] &= ~(0b00000001 << y);
			} else if((neighboors==2 || neighboors==3) && gval(x,y)==1){
				//     2  3  
			} else if(neighboors==3 && gval(x,y)==0){
				//      3   
				_chr[x] |= (0b00000001 << y);
			}
		}
	}
	
	for (tmp=0;tmp<5;tmp++){
		chr[tmp] = _chr[tmp];
	}
	
}

void randomize_field(void){
	unsigned char x;
	for (x=0; x<5; x++){
		chr[x] = my_rand();
	}
}

unsigned char gval(signed char x, signed char y){
/*
	if (x<0) x = 5+x%5;
	if (x>4) x = x%5;
	if (y<0) y = 8+y%8;
	if (y>7) y = y%8;
*/
	if (x<0) return 0;
	if (x>4) return 0;
	if (y<0) return 0;
	if (y>6) return 0;
	return (chr[x] >> y) & 0b00000001;
}

void display_frame(void) {
	unsigned char x;
	unsigned int i;
	for (i=0;i<frame_length;i++){
		for (x=0;x<5;x++){
			PORTB = ~ 0b00000000;
			PORTA = (0b00000001 << x); //    x  
			PORTB = ~ chr[x];
		}
	}
}

void clear_field(void) {
	unsigned int x;
	for (x=0;x<5;x++){
		PORTB = ~ 0b00000000;
		PORTA = (0b00000001 << x); //    x  
		PORTB = ~ 0b11111111;
		delay_ms(50);
	}
}

unsigned int seed = 1;

unsigned int my_rand( void ){
    if ( seed & 1 ) {
        seed >>= 1;
        seed ^= 0x8805;
    } else {
        seed >>= 1;
    }
    return seed;
}
